/* Ostern ist der erste Sonntag nach dem ersten Vollmond, der am
   21. Maerz oder spaeter erscheint (nicht voellig korrekt, da der
   wahre Stichtermin der Fruehlingsanfang ist) epact < 0 fuer year =
   9006. Siehe z. B.: D. Knuth: The Art of Computer Programming, Vol
   1, Fundamental Algorithms, 2nd Ed., p. 155-156  */ 
/* Zur Geschichte siehe http://www.nabkal.de/ostern.html */

#include <stdio.h>


void ostern_knuth(char *, int); 

int main() {
  char s[80];
  int year;
  while(1) {
    printf(" : ");
    scanf("%d", &year);
    ostern_knuth(s,year);
    printf("%s\n",s);
  } 
}



void ostern_knuth(char *date, int year) {  
  int golden, century;
  int x, z;
  int day, epacte, vollmond, ostern;

  century = year/100  + 1;   /* z.B. 1932 --> 20. Jahrhundert) */

  x =  3 * century/4 - 12;
            /* Anzahl der Jahrhundertzahlen ohne 29. Feb. seit 1582
               1600-->0,1700-->1,1800-->2,1900-->3,2000-->3,2100-->4 */

  day     = (5 * year/4 - x + 4) % 7;/* Wochentag des 21. Maerz: 0 = Sonntag */
            /*   5*year/4 = year + year/4   */

  golden = year % 19 + 1;           
            /* Goldene Zahl: Jahr in Mondzyklus = 19 Jahre */

  z = (8 * century + 5)/25 - 5;
            /* Korrektur ?? Ab 1600:// 0,0,0,1,1,1,2,2,2,3,3,3  */

  epacte   = (11 * golden + 20 + z - x) % 30;
           /* Epakte = Alter des Mondes bei Jahresbeginn 
                     = Anzahl der Tage seit letztem Neumond */

  if ( epacte < 0 ) { 
    epacte += 30; printf("Funny epacte - year: %d\n",year); 
  }
  /*  >= 0 mod 30  */

  if ( (epacte == 25 && golden > 11) || epacte == 24 ) 
    epacte++;

  vollmond = 44 - epacte;      /* 44 Tage = 1 1/2 Monate */
                               /* 15 <= vollmond <= 44 */
  if ( vollmond < 21 ) 
    vollmond += 30;            /* 21 <= vollmond <= 50 */

  ostern = vollmond + 7 - ((day + vollmond) % 7);

  if ( ostern > 31 ) 
    sprintf(date,"%2d.April.%4d", ostern - 31,year);
  else
    sprintf(date,"%2d.Maerz.%4d", ostern,   year);
}
